@@ -1,6 +1,7 @@
 # __init__.py
+import sys, inspect, glob
 from os import pardir, path
-import sys, inspect
+from pathlib import Path
 
 pkg_path = path.dirname( path.abspath(__file__) )
 bycon_lib_path = path.join( pkg_path, "lib" )
@@ -10,11 +11,14 @@ from aggregator_utils import *
 from args_parsing import *
 from cgi_parsing import *
 from data_retrieval import *
+from dataset_parsing import *
 from datatable_utils import *
 from handover_generation import *
 from interval_utils import *
 from filter_parsing import *
 from geomap_utils import *
+from helpers import *
+from plot_utils import *
 from publication_utils import *
 from query_execution import *
 from query_generation import *
@@ -25,13 +29,15 @@ from service_utils import *
 from export_file_generation import *
 from variant_parsing import *
 
-c_f = Path( path.join( pkg_path, "config", "config.yaml" ) )
+c_f = Path( path.join( pkg_path, "config.yaml" ) )
 config = load_yaml_empty_fallback( c_f )
+conf_dir = path.join( pkg_path, "config")
 byc = initialize_bycon(config)
-d_f = Path( path.join( pkg_path, "config", "beacon_defaults.yaml" ) )
+d_f = Path( path.join( conf_dir, "beacon_defaults.yaml" ) )
 byc.update({"beacon_defaults": load_yaml_empty_fallback( d_f ) })
-for d_k, d_v in byc["beacon_defaults"]["defaults"].items():
+defaults = byc["beacon_defaults"].get("defaults", {})
+for d_k, d_v in defaults.items():
     byc.update( { d_k: d_v } )
 
-read_bycon_definition_files(byc)
+read_bycon_definition_files(conf_dir, byc)
 cgi_parse_query(byc)
