@@ -1,6 +1,6 @@
 from os import environ, path, pardir
-import inspect, json
-import random
+import inspect, json, random
+from pathlib import Path
 from pymongo import MongoClient
 from bson import json_util
 
@@ -12,13 +12,15 @@ from args_parsing import *
 from cgi_parsing import *
 from cytoband_utils import translate_reference_ids
 from data_retrieval import *
+from dataset_parsing import select_dataset_ids
 from datatable_utils import export_datatable_download
 from export_file_generation import *
 from handover_generation import dataset_response_add_handovers, query_results_save_handovers, dataset_results_save_handovers
 from interval_utils import generate_genomic_intervals, interval_counts_from_callsets
+from plot_utils import histoplot_svg_generator
 from query_execution import execute_bycon_queries, process_empty_request, mongo_result_list
-from query_generation import  initialize_beacon_queries, paginate_list, replace_queries_in_test_mode, set_pagination_range
-from read_specs import read_bycon_configs_by_name,read_local_prefs
+from query_generation import  generate_queries, initialize_beacon_queries, paginate_list, set_pagination_range
+from read_specs import load_yaml_empty_fallback, read_bycon_configs_by_name, read_bycon_definition_files, read_local_prefs
 from response_remapping import *
 from schema_parsing import *
 
@@ -48,13 +50,14 @@ def initialize_bycon(config):
 
 def beacon_data_pipeline(byc, entry_type):
 
-    initialize_service(byc, entry_type)
+    initialize_bycon_service(byc, entry_type)
 
     run_beacon_init_stack(byc)
     return_filtering_terms_response(byc)
     run_result_sets_beacon(byc)
     update_meta_queries(byc)
     query_results_save_handovers(byc)
+    check_computed_histoplot_delivery(byc)
     check_computed_interval_frequency_delivery(byc)
     check_switch_to_count_response(byc)
     check_switch_to_boolean_response(byc)
@@ -62,29 +65,71 @@ def beacon_data_pipeline(byc, entry_type):
 
 ################################################################################
 
-def initialize_service(byc, service=False):
+def run_beacon_init_stack(byc):
+
+    select_dataset_ids(byc)
+    if len(byc["dataset_ids"]) < 1:
+        print_text_response("No existing dataset_id - please check dataset_definitions")
+
+    create_empty_beacon_response(byc)
+    initialize_beacon_queries(byc)
+    generate_genomic_intervals(byc)
+    response_collect_errors(byc)
+    cgi_break_on_errors(byc)
+
+    return byc
+
+################################################################################
+
+def initialize_bycon_service(byc, service=False):
 
     """For consistency, the name of the local configuration file should usually
     correspond to the calling main function. However, an overwrite can be
     provided."""
 
-    defs = byc["beacon_defaults"]
+    defs = byc.get("beacon_defaults", {})
+    b_e_d = defs.get("entity_defaults", {})    
     form = byc["form_data"]
 
     frm = inspect.stack()[1]
-    mod = inspect.getmodule(frm[0])
-    sub_path = path.dirname( path.abspath(mod.__file__) )
-
     if service is False:
         service = frm.function
 
     # TODO - streamline
-    s_a_s = byc["beacon_mappings"]["service_aliases"]
+    s_a_s = byc["beacon_mappings"].get("service_aliases", {})
 
     if service in s_a_s:
         service = s_a_s[service]
 
-    read_local_prefs( service, sub_path, byc )
+    service = decamelize(service)
+
+    mod = inspect.getmodule(frm[0])
+    if mod is not None:
+        """
+        Here we allow the addition of additional configuration files, necessary
+        for options beyond basic library use. Files are read in as
+        1. from a `config` directory in the parent directory of the executed script
+            * this e.g. overwrites the config options
+        2. from a config directory inside the script directory
+            * this is usually used to provide script-specific parameters (`service_defaults`...)
+        """
+        sub_path = path.dirname( path.abspath(mod.__file__) )
+
+        conf_dir = path.join( sub_path, "local" )
+        if path.isdir(conf_dir):
+            c_f = Path( path.join( conf_dir, "config.yaml" ) )
+            if path.isfile(c_f):
+                byc.update({"config": load_yaml_empty_fallback( c_f ) })
+            d_f = Path( path.join( conf_dir, "beacon_defaults.yaml" ) )
+            if path.isfile(d_f):
+                byc.update({"beacon_defaults": load_yaml_empty_fallback( d_f ) })
+                defaults = byc["beacon_defaults"].get("defaults", {})
+                for d_k, d_v in defaults.items():
+                    byc.update( { d_k: d_v } )
+            read_bycon_definition_files(conf_dir, byc)
+
+        read_local_prefs( service, sub_path, byc )
+
     get_bycon_args(byc)
     args_update_form(byc)
 
@@ -94,12 +139,13 @@ def initialize_service(byc, service=False):
         for d_k, d_v in conf["defaults"].items():
             byc.update( { d_k: d_v } )
 
-    if service in defs:
-        for d_k, d_v in defs[ service ].items():
+    if service in b_e_d:
+        for d_k, d_v in b_e_d[ service ].items():
             byc.update( { d_k: d_v } )
 
     # update response_entity_id from path
     update_entity_ids_from_path(byc)
+
     # update response_entity_id from form
     update_requested_schema_from_request(byc)
     set_response_entity(byc)
@@ -175,7 +221,6 @@ def update_entity_ids_from_path(byc):
         "response_entity_id": p_r_m.get(res_p_id, byc["response_entity_path_id"])
     })
 
-
     return byc
 
 ################################################################################
@@ -208,19 +253,6 @@ def set_response_entity(byc):
 
 ################################################################################
 
-def run_beacon_init_stack(byc):
-
-    initialize_beacon_queries(byc)
-    generate_genomic_intervals(byc)
-    create_empty_beacon_response(byc)
-    replace_queries_in_test_mode(byc)
-    response_collect_errors(byc)
-    cgi_break_on_errors(byc)
-
-    return byc
-
-################################################################################
-
 def run_result_sets_beacon(byc):
 
     sr_r = byc["service_response"]["response"]
@@ -232,24 +264,17 @@ def run_result_sets_beacon(byc):
     for i, r_set in enumerate(sr_r["result_sets"]):
 
         ds_id = r_set["id"]
+        generate_queries(byc, ds_id)
 
-        execute_bycon_queries( ds_id, byc )
+        r_set, r_s_res = populate_result_set(r_set, byc)
 
-        # Special check-out here since this forces a single handover
-        check_plot_responses(ds_id, byc)
-
-        r_set.update({ "results_handovers": dataset_response_add_handovers(ds_id, byc) })
-        r_s_res = retrieve_data(ds_id, byc)
-
-        r_set_update_counts(r_set, r_s_res, byc)
         if r_set["results_count"] < 1:
             continue
 
-        # TODO: This condition avoids the "range on range" if previously set for a handover
-        # query (by accessioinId)
+        # TODO: This condition avoids the "range on range" if previously set for
+        # a handover query (by accessioinId)
         if not "range" in byc["pagination"]:
-            set_pagination_range(len(r_s_res), byc)
-        
+            set_pagination_range(len(r_s_res), byc)        
         if test_truthy( byc["form_data"].get("paginate_results", True) ):
             r_s_res = paginate_list(r_s_res, byc)
 
@@ -276,16 +301,40 @@ def run_result_sets_beacon(byc):
 
 ################################################################################
 
+def populate_result_set(r_set, byc):
+
+    ds_id = r_set["id"]
+    execute_bycon_queries( ds_id, byc )
+
+    # # Special check-out here since this forces a single handover
+    # check_plot_responses(ds_id, byc)
+
+    r_set.update({ "results_handovers": dataset_response_add_handovers(ds_id, byc) })
+    r_s_res = retrieve_data(ds_id, byc)
+
+    r_set_update_counts(r_set, r_s_res, byc)
+
+    return r_set, r_s_res
+
+################################################################################
+
 def r_set_update_counts(r_set, r_s_res, byc):
 
     ds_id = r_set["id"]
+    ds_res = byc["dataset_results"][ds_id]
 
     r_set.update({"results_count": 0})
 
     for c, c_d in byc["config"]["beacon_counts"].items():
-        if c_d["h->o_key"] in byc["dataset_results"][ds_id]:
-            r_c = byc["dataset_results"][ds_id][ c_d["h->o_key"] ]["target_count"]
-            r_set["info"]["counts"].update({ c: r_c })
+        h_o_k = c_d.get("h->o_key", None)
+        if h_o_k is None:
+            continue
+        if h_o_k not in ds_res:
+            continue
+        r_c = ds_res[ h_o_k ]["target_count"]
+        s_c = len(list(set(ds_res[ h_o_k ]["target_values"])))
+        r_set["info"]["counts"].update({ c: r_c })
+
     if byc["empty_query_all_count"]:
         r_set.update({"results_count": byc["empty_query_all_count"] })
     elif isinstance(r_s_res, list):
@@ -311,13 +360,15 @@ def response_meta_add_request_summary(r, byc):
         return r
 
     r_rcvd_rs = r["meta"]["received_request_summary"]
+    defs = byc.get("beacon_defaults", {})
+    b_e_d = defs.get("entity_defaults", {})
 
     form = byc["form_data"]
 
     r_rcvd_rs.update({
         "filters": byc.get("filters", []), 
         "pagination": byc.get("pagination", {}),
-        "api_version": byc["beacon_defaults"]["info"].get("api_version", "v2")
+        "api_version": b_e_d["info"].get("api_version", "v2")
     })
 
     for p in ["include_resultset_responses", "requested_granularity"]:
@@ -351,7 +402,11 @@ def update_meta_queries(byc):
 
 def create_empty_beacon_response(byc):
 
-    r, e = instantiate_response_and_error(byc, byc["response_entity"]["response_schema"])
+    r_s = byc["response_entity"].get("response_schema", None)
+    if r_s is None:
+        return byc
+
+    r, e = instantiate_response_and_error(byc, r_s)
     response_update_meta(r, byc)
 
     try:
@@ -359,7 +414,7 @@ def create_empty_beacon_response(byc):
     except KeyError:
         pass
 
-    if "beaconResultsetsResponse" in byc["response_entity"]["response_schema"]:
+    if "beaconResultsetsResponse" in r_s:
 
         # TODO: stringent definition on when this is being used
         r_set = object_instance_from_schema_name(byc, "beaconResultsets", "definitions/ResultsetInstance/properties")
@@ -430,7 +485,8 @@ def check_plot_responses(ds_id, byc):
 
 def check_cnvhistogram_plot_response(ds_id, byc):
 
-    ds_h_o = byc["dataset_definitions"][ ds_id ]["handoverTypes"]
+    h_o_types = byc["handover_definitions"]["h->o_types"]
+    ds_h_o = byc["dataset_definitions"][ ds_id ].get("handoverTypes", h_o_types.keys())
 
     if "cnvhistogram" not in ds_h_o:
         return
@@ -483,19 +539,17 @@ def response_update_meta(r, byc):
     response_meta_set_config_defaults(r, byc)
     response_meta_set_entity_values(r, byc)
     response_meta_add_request_summary(r, byc)
-    if byc["test_mode"] is True:
-        r["meta"].update({"test_mode": byc["test_mode"]})
-    else:
-        r["meta"].pop("test_mode", None)
+    r["meta"].update({"test_mode": byc.get("test_mode", False)})
 
 ################################################################################
 
 def response_meta_set_info_defaults(r, byc):
 
-    # , "create_date_time", "update_date_time"
+    defs = byc.get("beacon_defaults", {})
+    b_e_d = defs.get("entity_defaults", {})
 
     for i_k in ["api_version", "beacon_id"]:
-        r["meta"].update({ i_k: byc["beacon_defaults"]["info"].get(i_k, "") })
+        r["meta"].update({ i_k: b_e_d["info"].get(i_k, "") })
 
     return r
 
@@ -560,7 +614,28 @@ def response_add_error(byc, code=200, message=False):
         return byc
 
     e = { "error_code": code, "error_message": message }
-    byc["error_response"]["error"].update(e)
+    byc["error_response"].update({"error": e})
+
+    return byc
+
+################################################################################
+
+def response_add_warnings(byc, message=False):
+
+    if message is False:
+        return byc
+    if len(str(message)) < 1:
+        return byc
+
+    if not "service_response" in byc:
+        return byc
+
+    if not "info" in byc["service_response"]:
+        byc["service_response"].update({"info": {}})
+    if not "warnings" in byc["service_response"]:
+        byc["service_response"]["info"].update({"warnings": []})
+
+    byc["service_response"]["info"]["warnings"].append(message)
 
     return byc
 
@@ -597,10 +672,27 @@ def collations_set_delivery_keys(byc):
 
 def populate_service_response( byc, results):
 
+    byc["service_response"].update({
+        "response_summary": {
+            "exists": False,
+            "num_total_results": 0
+        }
+    })
+
     populate_service_header(byc, results)
     populate_service_response_counts(byc)
+
+    if len(results) > 0:
+        byc["service_response"]["response_summary"].update({
+            "exists": True,
+            "num_total_results": len(results)
+        })
+
     byc["service_response"]["response"].update({"results": results })
 
+    check_switch_to_count_response(byc)
+    check_switch_to_boolean_response(byc)
+
     return byc
 
 ################################################################################
@@ -752,6 +844,51 @@ def create_filters_resource_response(collation_types, byc):
 
 ################################################################################
 
+def check_computed_histoplot_delivery(byc):
+
+    if byc["output"] not in ["histoplot", "cnvhistogram"]:
+        return byc
+
+    p_r = byc["pagination"]
+    interval_sets = []
+
+    for ds_id in byc["dataset_results"].keys():
+
+        ds_results = byc["dataset_results"][ds_id]
+
+        if not "callsets._id" in ds_results:
+            continue
+
+        cs_r = ds_results["callsets._id"]
+
+        mongo_client = MongoClient()
+        cs_coll = mongo_client[ ds_id ][ "callsets" ]
+        q_vals = cs_r["target_values"]
+        r_no = len(q_vals)
+        if r_no > p_r["limit"]:
+            q_vals = paginate_list(q_vals, byc)
+
+        cs_cursor = cs_coll.find({"_id": {"$in": q_vals }, "variant_class": { "$ne": "SNV" } } )
+        intervals, cnv_cs_count = interval_counts_from_callsets(cs_cursor, byc)
+        # print(f"{intervals[88]}")
+
+        i_set = {
+            "dataset_id": ds_id,
+            "group_id": ds_id,
+            "label": "Search Results",
+            "sample_count": cnv_cs_count,
+            "interval_frequencies": []
+        }
+
+        for intv_i, intv in enumerate(intervals):
+            i_set["interval_frequencies"].append(intv.copy())
+
+        interval_sets.append(i_set)
+
+    histoplot_svg_generator(byc, interval_sets)
+
+################################################################################
+
 def check_computed_interval_frequency_delivery(byc):
 
     if not "frequencies" in byc["output"]:
@@ -790,6 +927,7 @@ def check_computed_interval_frequency_delivery(byc):
     cs_cursor = cs_coll.find({"_id": {"$in": q_vals }, "variant_class": { "$ne": "SNV" } } )
 
     intervals, cnv_cs_count = interval_counts_from_callsets(cs_cursor, byc)
+
     for intv in intervals:
         v_line = [ ]
         v_line.append("query_result")
